---
topology_name: jenkins-slave
resource_groups:
- resource_group_name: beaker-slaves
  resource_group_type: beaker
  resource_definitions:
  - role: bkr_server
    job_group: "{{ job_group }}"
    whiteboard: "{{ arch }} slave for multiarch testing"
    recipesets:
    - distro: RHEL-7.4
      arch: "{{ arch }}"
      variant: Server
      count: 1
